// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package jobqueue

import (
	"encoding/json"
	"github.com/robfig/cron/v3"
	"github.com/stretchr/testify/assert"
	"io/ioutil"
	"os"
	"testing"
	"time"
)

func TestCreateJobAndSchedulerFromInput(t *testing.T) {

	time10s := 10 * time.Second
	time2s := 5 * time.Minute

	tests := []struct {
		name      string
		input     JobPersistence
		wantJob   GenericJob
		wantSched Scheduler
		wantErr   bool
	}{
		{
			name: "Shell Runnable and Interval Scheduler",
			input: JobPersistence{
				ID:         "1",
				Priority:   1,
				Timeout:    &time10s,
				MaxRetries: 3,
				RetryDelay: &time2s,
				Runnable: RunnableImport{
					Type: "Shell",
					Data: map[string]any{"ScriptPath": "script.sh"},
				},
				Scheduler: SchedulerPersistence{
					Type:     "Interval",
					Interval: 1 * time.Minute,
				},
			},
			wantJob:   GenericJob(&Job[ShellResult]{ /* Initialization */ }),
			wantSched: &IntervalScheduler{Interval: 1 * time.Minute},
			wantErr:   false,
		},
		{
			name: "Shell Runnable and Cron Scheduler",
			input: JobPersistence{
				ID:       "1",
				Priority: 1,
				Runnable: RunnableImport{
					Type: "Shell",
					Data: map[string]any{"ScriptPath": "script.sh"},
				},
				Scheduler: SchedulerPersistence{
					Type: "Cron",
					Spec: "* * * * * *",
				},
			},
			wantJob:   GenericJob(&Job[ShellResult]{ /* Initialization */ }),
			wantSched: &CronScheduler{ /* Initialization */ },
			wantErr:   false,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			gotJob, gotSchedule, err := CreateJobAndSchedulerFromPersistence(tt.input, nil)
			if gotSchedule != nil {
				if gotSchedule.GetType() == "Cron" {
					cronInst := cron.New(cron.WithSeconds())
					gotSchedule.(*CronScheduler).cron = cronInst
					cronInst.Start()
					defer cronInst.Stop()

				}
			}

			if (err != nil) != tt.wantErr {
				t.Errorf("CreateJobAndSchedulerFromPersistence() error = %v, wantErr %v", err, tt.wantErr)
				return
			}

			assert.Equal(t, JobID(tt.input.ID), gotJob.GetID(), "Job ID mismatch")
			assert.Equal(t, Priority(tt.input.Priority), gotJob.GetPriority(), "Job Priority mismatch")

			assert.Equal(t, tt.input.Scheduler.Type, gotSchedule.GetType(), "Scheduler Type mismatch")

		})
	}
}

func TestJobPersistence_MarshalUnmarshalJSON(t *testing.T) {
	timeRef := time.Now().Round(time.Second) // Runden, um Genauigkeitsverlust zu vermeiden
	timeRefString := timeRef.Format(time.RFC3339)

	time5m := 5 * time.Minute
	time10s := 10 * time.Second
	time30s := 30 * time.Second
	time1Hour := 1 * time.Hour

	time10sString := "10s"
	time30sString := "30s"
	time5m0sString := "5m0s"
	time1h0m0sString := "1h0m0s"
	emptyString := ""

	tests := []struct {
		name     string
		job      JobPersistence
		expected string
	}{
		{
			name: "All fields present",
			job: JobPersistence{
				Timeout:          &time5m,
				RetryDelay:       &time10s,
				PauseUntil:       &timeRef,
				TimeoutString:    &time5m0sString,
				RetryDelayString: &time10sString,
				PauseUntilString: &timeRefString,
			},
			expected: ` {"id":"","description":"","priority":0,"maxRetries":0,"runnable":{"type":""},"scheduler":{"type":""},"pause":false,"pauseReason":"","timeout":"5m0s","retryDelay":"10s","pauseUntil":"` + timeRefString + `","stats":{"jobId":"","runCount":0,"successCount":0,"errorCount":0,"timeMetrics":{"avg":0,"max":0,"min":0,"total":0},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"}`,
		},
		{
			name: "Nil pauseUntil",
			job: JobPersistence{
				Timeout:          &time1Hour,
				RetryDelay:       &time30s,
				PauseUntil:       nil,
				TimeoutString:    &time1h0m0sString,
				RetryDelayString: &time30sString,
				PauseUntilString: &emptyString,
			},
			expected: `{"id":"","description":"","priority":0,"maxRetries":0,"runnable":{"type":""},"scheduler":{"type":""},"pause":false,"pauseReason":"","timeout":"1h0m0s","retryDelay":"30s","pauseUntil":"","stats":{"jobId":"","runCount":0,"successCount":0,"errorCount":0,"timeMetrics":{"avg":0,"max":0,"min":0,"total":0},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"}`,
		},
		{
			name: "Nil timeout and retryDelay",
			job: JobPersistence{
				Timeout:          nil,
				RetryDelay:       nil,
				PauseUntil:       nil,
				TimeoutString:    &emptyString,
				RetryDelayString: &emptyString,
				PauseUntilString: &emptyString,
			},
			expected: `{"id":"","description":"","priority":0,"maxRetries":0,"runnable":{"type":""},"scheduler":{"type":""},"pause":false,"pauseReason":"","timeout":"","retryDelay":"","pauseUntil":"","stats":{"jobId":"","runCount":0,"successCount":0,"errorCount":0,"timeMetrics":{"avg":0,"max":0,"min":0,"total":0},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"},"createdAt":"0001-01-01T00:00:00Z","updatedAt":"0001-01-01T00:00:00Z"}`,
		},
	}

	for _, tc := range tests {
		t.Run(tc.name, func(t *testing.T) {
			// MarshalJSON testen
			data, err := json.Marshal(tc.job)

			assert.NoError(t, err)
			assert.JSONEq(t, tc.expected, string(data))

			var job JobPersistence
			err = json.Unmarshal(data, &job)
			assert.NoError(t, err)
			assert.Equal(t, tc.job.Timeout, job.Timeout)
			assert.Equal(t, tc.job.RetryDelay, job.RetryDelay)
			if tc.job.PauseUntil != nil {
				assert.Equal(t, *tc.job.PauseUntil, *job.PauseUntil)
			} else {
				assert.Nil(t, job.PauseUntil)
			}
		})
	}
}

func TestReadJsonFile(t *testing.T) {
	testContent := `[{"id": "1", "priority": 1}]`
	tempFile, err := ioutil.TempFile("", "test.json")
	if err != nil {
		t.Fatal(err)
	}
	defer os.Remove(tempFile.Name())

	if _, err := tempFile.Write([]byte(testContent)); err != nil {
		t.Fatal(err)
	}
	tempFile.Close()

	jobs, err := ReadJsonFile(tempFile.Name())
	if err != nil {
		t.Fatal(err)
	}

	if len(jobs) != 1 || jobs[0].ID != "1" || jobs[0].Priority != 1 {
		t.Errorf("Expected job with ID '1' and priority 1, got %+v", jobs)
	}
}

func TestReadYAMLFile(t *testing.T) {
	testContent := `- id: "1"
  priority: 1
`
	tempFile, err := ioutil.TempFile("", "test.yaml")
	if err != nil {
		t.Fatal(err)
	}
	defer os.Remove(tempFile.Name())

	if _, err := tempFile.Write([]byte(testContent)); err != nil {
		t.Fatal(err)
	}
	tempFile.Close()

	jobs, err := ReadYAMLFile(tempFile.Name())
	if err != nil {
		t.Fatal(err)
	}

	if len(jobs) != 1 || jobs[0].ID != "1" || jobs[0].Priority != 1 {
		t.Errorf("Expected job with ID '1' and priority 1, got %+v", jobs)
	}
}

func TestJobPersistenceUnmarshalJSON(t *testing.T) {
	testCases := []struct {
		name      string
		jsonInput string
		expected  *time.Time
		wantErr   bool
	}{
		{
			name:      "RFC3339 Format",
			jsonInput: `{"pauseUntil":"2023-11-15T09:01:00Z"}`,
			expected:  parseTimeForTesting(t, "2023-11-15T09:01:00Z"),
			wantErr:   false,
		},
		{
			name:      "Date and Time",
			jsonInput: `{"pauseUntil":"2023-11-15T09:01"}`,
			expected:  parseTimeForTesting(t, "2023-11-15T09:01"),
			wantErr:   false,
		},
		{
			name:      "Only Date",
			jsonInput: `{"pauseUntil":"2023-11-15"}`,
			expected:  parseTimeForTesting(t, "2023-11-15"),
			wantErr:   false,
		},
		{
			name:      "Invalid Format",
			jsonInput: `{"pauseUntil":"15. November 2023"}`,
			expected:  nil,
			wantErr:   true,
		},
	}

	for _, tc := range testCases {
		t.Run(tc.name, func(t *testing.T) {
			var jp JobPersistence
			err := json.Unmarshal([]byte(tc.jsonInput), &jp)

			if tc.wantErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tc.expected, jp.PauseUntil)
			}
		})
	}
}
